% function [PkPower_dBm, Fmax, Measure] = PeakMeasurement(Signal, Resolution, VBW, RBW, R)
%
% This function provides the Peak measurement as performed by a spectrum
% analyzer. The FCC requirements are 1MHz < RBW < 50MHz and VBW >= 3MHz
%   Resolution : temporal resolution of Signal    [s]
%   VBW        : Video Bandwidth                  [Hz]
%   RBW        : Resolution Bandwidth             [Hz]
%   R          : Characteristic Antenna Impedance [OHm]
%
% 'PkPower_dBm' is the max of the performed measurements, 'Fmax' is the
% corresponding frequency.
% 
% Author: samuel.dubouloz@cea.fr, 
%         CEA-LETI - May, 2005
%

function [PkPower_dBm, Fmax] = PeakMeasurement(Signal, Resolution, VBW, RBW, R)

%% Default values, Warning and Error Management
if nargin < 2, error('some paramters are missing')
elseif nargin == 2, R = 50; VBW = 5e6; RBW = 3e6;
elseif nargin == 3, R = 50; VBW = 5e6;
elseif nargin == 4, R = 50; end
if(RBW ~= 3e6 || VBW < 3e6), warning('!!! non FCC compliant conditions !!! '), end
if(VBW < RBW), warning('!!! The VBW should be lower than RBW !!!'), end

%% Research of Fmax
dt = Resolution;
N = length(Signal);
PsdSig = abs(fft(Signal));
PsdSig = PsdSig(1:round(end/2));
[v, index] = max(PsdSig);
clear PsdSig;
Fmax = index/(N*dt);
   
%% RBW Filtering (Band Pass Filter)
LowFreq = 2 * dt * (Fmax - RBW/2);
UpFreq  = 2 * dt * (Fmax + RBW/2);
[num, den] = butter(2, [LowFreq UpFreq]);
RBW_Signal = filter(num, den, Signal);

%% VBW filtering (Low Pass Filter)
upper_freq = 2 * dt * VBW;
[num, den] = butter(2, upper_freq,'low');
VBW_Signal = filter(num, den, abs(RBW_Signal).^2);
clear RBW_Signal

%% Power Calculation
Ppk = max(VBW_Signal)/R;
PkPower_dBm = 10*log10(Ppk)+30;
